local _      = require 'libs/underscore'
local graphics = require 'graphics'
local sounds = require 'sound'
local timeout = require 'timeout'
local rect = require 'rect'


local scale = 0.0390625 

local game = {
    tiles = {},

    --Link to the top row of tiles (top mean the tiles with the lowest y value)
    --gets incermented when bottom tiles are put on top
    scrollReset = 0,
    hazards = {},

    hazard_timer = 2,

    lost = false,
    win = false

}

game.init = function ()
    sounds.cleanup()

    game.scrollspeed = -5

    love.graphics.setLineWidth( scale )
    for i=-4,5 do 
        local row = {}
        local tile 

        tile = rect.create("", -10, i*5, 5, 5)
        tile.graphic = 'tent_blue'
        tile.name = 'tent'
        _.push(row, tile)

        tile = rect.create("", -5, i*5, 5, 5)
        tile.graphic = 'land_grass04'
        tile.name = 'grass'
        _.push(row, tile)
        
        tile = rect.create("", 0, i*5, 5, 5)
        tile.graphic = 'land_grass04'
        tile.name = 'grass'
        _.push(row, tile)

        tile = rect.create("", 5, i*5, 5, 5)
        tile.graphic = 'road_asphalt21'
        tile.name = 'road'
        _.push(row, tile)

        tile = rect.create("", 10, i*5, 5, 5)
        tile.graphic = 'road_asphalt22'
        tile.name = 'road'
        _.push(row, tile)

        tile = rect.create("", 15, i*5, 5, 5)
        tile.graphic = 'road_asphalt23'
        tile.name = 'road'
        _.push(row, tile)

        tile = rect.create("", 20, i*5, 5, 5)
        tile.graphic = 'land_grass04'
        tile.name = 'grass'
        _.push(row, tile)

        tile = rect.create("", 25, i*5, 5, 5)
        tile.graphic = 'land_grass04'
        tile.name = 'grass'
        _.push(row, tile)

        tile = rect.create("", 30, i*5, 5, 5)
        tile.graphic = 'land_grass04'
        tile.name = 'grass'
        _.push(row, tile)

        tile = rect.create("", 35, i*5, 5, 5)
        tile.graphic = 'tent_blue'
        tile.name = 'tent'
        _.push(row, tile)

        _.push(game.tiles, row)

    end


    game.player = rect.create("player", 11, 15, 5, 5)
    game.player.graphic = "car_yellow_2"
    game.player.topspeed = -5
    game.player.speed = game.player.topspeed
    game.player.updateBump()
    game.player.oil = false
    game.player.grass = false
    game.player.hide = false
    sounds.add("sfx/engine_small.ogg", game.player, true, .1, 1.1)


    game.tracker = rect.create("tracker", 12.5, 10, 2, 2)
    game.tracker.graphic = "cone_straight"
    game.tracker.v.y = game.scrollspeed

    game.sweeper = rect.create("sweeper", -10, 40, 5, 50)
    game.sweeper.graphic = "cone_straight"
    game.sweeper.v.y = game.scrollspeed
    game.sweeper.lost = false

    game.topsweeper = rect.create("topsweeper", -10, -15, 5, 50)
    game.topsweeper.graphic = "cone_straight"
    game.topsweeper.v.y = game.scrollspeed

    game.winblock = rect.create("win", -10, -4900, 5, 50)
    game.winblock.graphic = "cone_straight"
    game.winblock.win = false
    
    game.scrollspeedtimer = 1
    game.hazard_rate = 2
end

game.update_scrollspeed = function(speed)
    game.scrollspeed = speed
    game.tracker.v.y = speed
    game.sweeper.v.y = speed
    game.topsweeper.v.y = speed
    game.player.topspeed = speed*0.85
end


game.update_hazards = function(dt)
    if(game.win == false and game.lost == false) then
        game.hazard_timer = game.hazard_timer - dt
        if(game.hazard_timer < 0) then
            game.hazard_timer = game.hazard_rate + 0.3
            game.hazard_rate = game.hazard_rate*0.99
            game.add_random_hazard(game.tracker.p.y)
        end
    end
end


--check for hazards that need cleaned up a
game.cleanup_hazards = function()
    local good = _.select(game.hazards, function(haz)
        return haz.cleanup == false
    end)

    local rejects = _.select(game.hazards, function(haz)
        return haz.cleanup == true
    end)

    game.hazards = good

    rect.cleanupByName("hazard")
end

game.add_random_hazard = function(y)
    local hazards = {
        'rock1', 
        'rock2', 
        'rock3', 
        'oil', 
        'cone_straight', 
        'tree_large', 
        'tree_small', 
        'barrel_red', 
        'barrel_blue', 
        --'tires_red', 
        --'tires_white', 
        'tires_white_alt', 
        'tires_red_alt', 
        --'tribune_full', 
        --'tribune_empty', 
        --'barrier_red', 
        'barrier_red_race', 
        --'barrier_white', 
        --'barrier_white_race', 
        'car_red_1', 
        'car_blue_1', 
        'car_yellow_3', 
        'car_green_2'
    }
    local rand2 = love.math.random(#hazards)
    local graphic = hazards[rand2]
    --print(graphic)
    local h = graphics.imgs[graphic .. ".png"].h*scale
    local w = graphics.imgs[graphic .. ".png"].w*scale
    local x = 3 + love.math.random(15 - w) 
    local item = rect.create("hazard", x , y - 30  , h, w)
    item.graphic = graphic
    item.cleanup = false

    if (item.graphic == 'oil') then
        item.stuck = false
    end

    if (item.graphic == 'car_red_1' or 
        item.graphic == 'car_blue_1' or
        item.graphic == 'car_yellow_3' or
        item.graphic == 'car_green_2') then
        item.m = 1
        item.speed = (love.math.random()*6 + -2)*6
        sounds.add("sfx/engine_small.ogg", item, true, 1, 1)
        if(item.speed > 0) then
            item.r = 3.1415
        end
    elseif (item.graphic == 'tree_large' or 
        --item.graphic == 'rock1' or 
        --item.graphic == 'rock2' or 
        --item.graphic == 'rock3' or 
        item.graphic == 'tree_small') then
        item.m = 1000
        item.p.x = love.math.random(7)-5 + love.math.random(8)%2*22
        item.speed = 0
    elseif (item.graphic == 'barrier_red_race' or 
        item.graphic == 'barrier_white_race' or 
        item.graphic == 'barrier_white' or 
        item.graphic == 'barrier_red') then
        item.m = 1000
        item.speed = 0
        item.p.x = 3 + love.math.random(20 - item.w)
    elseif (item.graphic == 'cone_straight' or 
        item.graphic == 'barrel_blue' or 
        item.graphic == 'barrel_red' or 
        item.graphic == 'tires_white_alt' or 
        item.graphic == 'tires_red_alt') then
        item.m = 0.2
        item.b = 0.2
    else
        item.m = 0.5
        item.speed = 0
        item.p.x = 5 + love.math.random(15 - item.w)
    end

    _.push(game.hazards, item)
    game.cleanup_hazards()
end

game.add_named_hazard = function(name, x, y)
    local item = rect.create("hazard", x , y , 10, 10)
    item.v.x = 0
    item.v.y = 0
    if(name == "oil") then
        item.stuck = false
    end
    item.graphic = name
    item.cleanup = false
    item.updateBump()
    _.push(game.hazards, item)
end

game.updateScrollSpeed = function(dt)
    if(game.win or game.lost) then
        --slows the scrollspped down after win or lose 
        if(game.scrollspeed <  0) then
            game.scrollspeed = game.scrollspeed + dt*10
            game.update_scrollspeed(game.scrollspeed)
        else
            game.scrollspeed = 0
            game.update_scrollspeed(game.scrollspeed)
        end
    else
        --makes the scroll speed up little by little every frame
        game.scrollspeed = game.scrollspeed - dt*0.1
        game.update_scrollspeed(game.scrollspeed)
    end
end

game.updatePlayer = function(dt, a)
    game.player.oil = false
    game.player.grass = false
    if(game.win == false and game.lost == false) then
        game.player.f.y = game.player.speed*5 + a.y*20
        game.player.f.x = a.x*80
    end
    local np = 0.03*math.abs(game.player.v.y) + 0.9
    sounds.pitch(game.player, np)
    love.audio.setPosition(game.player.p.x, game.player.p.y, 0)
end

game.update = function(dt,a)
    rect.zeroForce()
    game.updateScrollSpeed(dt)
    game.update_hazards(dt)
    game.updatePlayer(dt, a)

    --This needs to go last as it updates the physics
    rect.update(dt)
    sounds.update()

    if(game.winblock.win == true) then game.win = true end
    if(game.sweeper.lost == true) then game.lost = true end
end

game.hidePlayer = function()
    game.player.hide = true
end

game.draw = function()
    _.each(game.tiles, 
        function(row) 
            _.each(row, function(item)item.draw() end) 
        end)

    local list = _.select(game.hazards,
        function(item) 
            return item.graphic == "oil"
        end)
        
    _.each(list, function(item)
        item.draw()
    end)

    list = _.select(game.hazards,
        function(item) 
            return item.graphic ~= "oil"
        end)
        
    _.each(list, function(item)
        item.draw()
    end)

    if(game.player.hide == false) then
        game.player.draw()
    end

    --rect.debugDraw();
    
end


game.reset = function()
    rect.reset()

    game.tiles = {}
    game.scrollReset = 0
    game.hazards = {}
    game.hazard_timer = 2
    game.lost = false
    game.win = false


    game.init()

end

game.setCar = function (car)
    game.player.hide = false
    game.player.graphic = car
    game.player.updateBump()

    
end

game.getRef = function ()
    local ref = {x = game.player.p.x + game.player.w/2, y = game.player.p.y + game.player.h/2 }
    return ref
end

game.getFocus = function ()
    local ref = game.tracker.p
    return ref
end

return game
