local _      = require 'libs/underscore'
local xml = require("libs/xmlSimple").newParser()
local graphics = {}

function parseSpritesheet(xmlFile, imageFile, named_array, list_array)
    local contents, size = love.filesystem.read( xmlFile )
    local parsed_xml = xml:ParseXmlText(contents)
    local img = love.graphics.newImage( imageFile)
    _.each(parsed_xml.TextureAtlas.SubTexture, function(item)
        local node = {}
        node.x = item['@x']
        node.y = item['@y']
        node.w = item['@width']
        node.h = item['@height']
        node.name = item['@name']
        node.quad = love.graphics.newQuad(node.x, node.y, node.w, node.h, img:getDimensions())
        node.draw = function(x, y, r, s)
            love.graphics.draw(img, node.quad, x, y, r, s)
        end
        named_array[item['@name']] = node
        _.push(list_array, node)
    end)
end

graphics.init = function()
    local carXmlFile = "graphics/spritesheet_vehicles.xml" 
    local carImg = "graphics/spritesheet_vehicles.png"
    local imgs = {}
    local car_list = {}

    parseSpritesheet(carXmlFile, carImg, imgs, car_list)
    graphics.cars = cars
    graphics.car_list = car_list

    local carXmlFile = "graphics/spritesheet_objects.xml" 
    local carImg = "graphics/spritesheet_objects.png"
    graphics.obj_list = {}
    parseSpritesheet(carXmlFile, carImg, imgs, graphics.obj_list)

    local carXmlFile = "graphics/spritesheet_tiles.xml" 
    local carImg = "graphics/spritesheet_tiles.png"
    graphics.tile_list = {}
    parseSpritesheet(carXmlFile, carImg, imgs, graphics.tile_list)

    graphics.imgs = imgs
end

return graphics
